package com.caimei365.user.controller;

import com.caimei365.user.model.ResponseJson;
import com.caimei365.user.model.dto.*;
import com.caimei365.user.model.vo.UserLoginVo;
import com.caimei365.user.service.LoginService;
import io.swagger.annotations.*;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * 登录API
 *
 * @author : Charles
 * @date : 2021/3/8
 */
@Api(tags="登录API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/user/login")
public class LoginApi {

    private final LoginService loginService;

    /**
     * 登录(用户名，密码)
     *
     * spi旧接口：/user/login
     *
     * @param loginPasswordDto {
     *                           mobileOrEmail 手机号或邮箱
     *                           password      密码
     *                           unionId       微信unionId
     *                         }
     * @return UserLoginVo
     */
    @ApiOperation("登录(用户名，密码)")
    @PostMapping("/password")
    public ResponseJson<UserLoginVo> passwordLogin(LoginPasswordDto loginPasswordDto) {
        return loginService.passwordLogin(loginPasswordDto);
    }

    /**
     * 微信授权登录(小程序)，用户数据存入Redis，key前缀：wxInfo:applets:
     *
     * spi旧接口：/club/authorization，小程序sessionKey(wxInfo)、PC端sessionKey(wxInfoMap)
     *
     * @param authAppletsDto {
     *                           code          微信授权code
     *                           encryptedData 微信加密数据
     *                           iv            加密算法的初始向量
     *                         }
     */
    @ApiOperation("微信授权登录(小程序)")
    @PostMapping("/auth/applets")
    public ResponseJson<UserLoginVo> appletsAuthorization(AuthAppletsDto authAppletsDto, @RequestHeader HttpHeaders headers) {
        String code = authAppletsDto.getCode();
        String encryptedData = authAppletsDto.getEncryptedData();
        String iv = authAppletsDto.getIv();
        return loginService.appletsAuthorization(code, encryptedData, iv, headers);
    }

    /**
     * 微信公众号授权链接(www)
     *
     * spi旧接口：/user/authorizationLink
     *
     * @param redirectUri 用于微信授权的中间页面
     * @param mode        授权方式：1静默授权，其他手动同意授权
     */
    @ApiOperation("微信公众号授权链接(www)")
    @ApiImplicitParams({
        @ApiImplicitParam(required = true, name = "redirectUri", value = "用于微信授权的中间页面"),
        @ApiImplicitParam(required = false, name = "mode", value = "授权方式：1静默授权，其他手动同意授权")
    })
    @GetMapping("/auth/link")
    public ResponseJson<String> getAuthorizationLink(String redirectUri, Integer mode) {
        return loginService.getAuthorizationLink(redirectUri, mode);
    }

    /**
     * 微信公众号授权登录(www)，用户数据存入Redis，key前缀：wxInfo:website:
     *
     * spi旧接口：/user/authorizationLogin，小程序sessionKey(wxInfo)、PC端sessionKey(wxInfoMap)
     *
     * @param code  微信code
     * @param state 安全认证
     * @param mode  1:静默授权,2:用户手动授权
     */
    @ApiOperation("微信公众号授权登录(www)")
    @ApiImplicitParams({
        @ApiImplicitParam(required = true, name = "code", value = "微信授权code"),
        @ApiImplicitParam(required = true, name = "state", value = "安全认证key"),
        @ApiImplicitParam(required = false, name = "mode", value = "1:静默授权,2:用户手动授权")
    })
    @GetMapping("/auth/website")
    public ResponseJson<UserLoginVo> websiteAuthorization(String code, String state, Integer mode, @RequestHeader HttpHeaders headers) {
        return loginService.websiteAuthorization(code, state, mode, headers);
    }

    /*
     * 微信扫码登录（www）
     *
     *   流程：1.前端页面中先引入JS文件（支持https）：http://res.wx.qq.com/connect/zh_CN/htmledition/js/wxLogin.js
     *        2.后端提供接口（/user/login/auth/parameters）返回生成二维码的参数：appid，scope，redirect_uri，其余参数由前端自定义，详细参考微信文档：https://developers.weixin.qq.com/doc/oplatform/Website_App/WeChat_Login/Wechat_Login.html
     *        3.前端实例化JS对象（用于生成二维码）： new WxLogin({self_redirect:true, id:"login_container", appid:"", scope:"", redirect_uri:"", state:"", style:"", href: ""});
     *        4.用户扫码：用户使用微信扫码授权后,微信服务器回调：/user/login/auth/scan?code=CODE&state=STATE，后端接口根据code和state换取access_token,并通过access_token获取用户信息，并存入Redis.
     *        5.前端通过轮询（/user/login/auth/result）监听是否扫码完成，完成后获取用户数据（Redis取出）。
     */

    /**
     * 获取生成微信二维码的参数
     *
     * spi旧接口：/user/toWechatLogin
     *
     * @return Map(appId,redirectUri,state)
     */
    @ApiOperation("获取生成微信二维码的参数")
    @GetMapping("/auth/parameters")
    public ResponseJson<Map<String, String>> getAuthParameters(){
        return loginService.getAuthParameters();
    }

    /**
     * 微信用户扫码,微信服务器回调
     *
     * spi旧接口：/user/wechatLogin
     *
     * @param code      微信code
     * @param state     安全认证key（第一步获取参数时自定义生成的uuid）
     */
    @ApiOperation("扫码后微信服务器回调")
    @ApiImplicitParams({
        @ApiImplicitParam(required = false, name = "code", value = "微信授权code"),
        @ApiImplicitParam(required = false, name = "state", value = "安全认证key")
    })
    @GetMapping("/auth/callback")
    public void qrCodeAuthScan(String code, String state) {
        loginService.qrCodeAuthScan(code, state);
    }

    /**
     * 校验扫码结果，用户数据存入Redis，key前缀：wxInfo:website:
     *
     * spi旧接口：/user/polling，小程序sessionKey(wxInfo)、PC端sessionKey(wxInfoMap)
     *
     * @param state   安全认证key（第一步获取参数时自定义生成的uuid）
     * @return UserLoginVo
     */
    @ApiOperation("校验扫码登录结果")
    @ApiImplicitParam(required = true, name = "state", value = "安全认证key")
    @GetMapping("/auth/scan/result")
    public ResponseJson<UserLoginVo> qrCodeAuthScanResult(String state){
        return loginService.qrCodeAuthScanResult(state);
    }

    /**
     * 微信扫码后,绑定机构账号
     *
     * spi旧接口：/user/bindOrganization
     *
     * @param scanBindDto {
     *                           mobileOrEmail 手机号或邮箱
     *                           password      密码
     *                           mobile        手机号
     *                           smsCode       手机验证码
     *                           unionId       微信unionId
     *                           linkName      联系人
     *                     }
     */
    @ApiOperation("微信扫码后绑定机构账号")
    @PostMapping("/auth/scan/bind")
    public ResponseJson<UserLoginVo> qrCodeAuthScanBind(ScanBindDto scanBindDto){
        return loginService.qrCodeAuthScanBind(scanBindDto);
    }

    /**
     * 运营人员邀请码授权登录
     *
     * spi旧接口：/club/invitationCode
     *
     * @param authInvitationDto {
     *                             invitationCode 邀请码
     *                             nickName       微信昵称
     *                             avatarUrl      微信头像(headimgurl)
     *                             unionId        微信unionId
     *                          }
     * @return UserLoginVo
     */
    @ApiOperation("运营人员邀请码授权登录")
    @PostMapping("/auth/invitation")
    public ResponseJson<UserLoginVo> invitationCodeLogin(AuthInvitationDto authInvitationDto) {
        String invitationCode = authInvitationDto.getInvitationCode();
        String nickName = authInvitationDto.getNickName();
        String avatarUrl = authInvitationDto.getAvatarUrl();
        String unionId = authInvitationDto.getUnionId();
        return loginService.invitationCodeLogin(invitationCode, nickName, avatarUrl, unionId);
    }

    /**
     * 运营人员绑定微信
     *
     * spi旧接口：/club/bindingWx
     *
     * @param authBindDto {
     *                         userId    要绑定的用户Id(userID)
     *                         mobile    手机号
     *                         smsCode   手机验证码(verificationCode)
     *                         unionId   微信unionId
     *                         nickName       微信昵称
     *                         avatarUrl      微信头像(headimgurl)
     *                     }
     * @return UserLoginVo
     */
    @ApiOperation("运营人员绑定微信")
    @PostMapping("/auth/bind")
    public ResponseJson<UserLoginVo> operationBindWeChat(AuthBindDto authBindDto) {
        return loginService.operationBindWeChat(authBindDto);
    }


}
